From 701414c56d4e6d80cf386c1b5e3d9976cd588871 Mon Sep 17 00:00:00 2001
From: Pavel Baturko <pabftk@gmail.com>
Date: Sun, 20 Jan 2013 22:05:37 +0000
Subject: [PATCH] Decode UCS2 SMS to UTF8

---
 src/host/layer23/src/mobile/app_mobile.c |    8 ++++++++
 src/host/layer23/src/mobile/gsm411_sms.c |   16 +++++++++++++---
 2 files changed, 21 insertions(+), 3 deletions(-)

diff --git a/src/host/layer23/src/mobile/app_mobile.c b/src/host/layer23/src/mobile/app_mobile.c
index 09ff785..d59ab56 100644
--- a/src/host/layer23/src/mobile/app_mobile.c
+++ b/src/host/layer23/src/mobile/app_mobile.c
@@ -376,6 +376,12 @@ static struct vty_app_info vty_info = {
 	.go_parent_cb = ms_vty_go_parent,
 };
 
+static uint8_t terminal_utf8_support = 0;
+uint8_t get_terminal_utf8_support()
+{
+	return terminal_utf8_support;
+}
+
 /* global init */
 int l23_app_init(int (*mncc_recv)(struct osmocom_ms *ms, int, void *),
 	const char *config_file, const char *vty_ip, uint16_t vty_port)
@@ -407,6 +413,8 @@ int l23_app_init(int (*mncc_recv)(struct osmocom_ms *ms, int, void *),
 		return rc;
 	printf("VTY available on port %u.\n", vty_port);
 
+	terminal_utf8_support = osmo_check_utf8_locale();
+
 	osmo_signal_register_handler(SS_GLOBAL, &global_signal_cb, NULL);
 	osmo_signal_register_handler(SS_L1CTL, &mobile_signal_cb, NULL);
 	osmo_signal_register_handler(SS_L1CTL, &gsm322_l1_signal, NULL);
diff --git a/src/host/layer23/src/mobile/gsm411_sms.c b/src/host/layer23/src/mobile/gsm411_sms.c
index 9de1259..9ba0a06 100644
--- a/src/host/layer23/src/mobile/gsm411_sms.c
+++ b/src/host/layer23/src/mobile/gsm411_sms.c
@@ -184,12 +184,20 @@ static int gsm340_rx_sms_deliver(struct osmocom_ms *ms, struct msgb *msg,
 	char *sms_file;
 	char vty_text[sizeof(gsms->text)], *p;
 	FILE *fp;
+	uint8_t text_in_utf8 = 0;
 
 	/* remove linefeeds and show at VTY */
 	strcpy(vty_text, gsms->text);
 	for (p = vty_text; *p; p++) {
 		if (*p == '\n' || *p == '\r')
 			*p = ' ';
+		else if ((uint8_t)*p > 0x7F) {
+			text_in_utf8 = 1;
+			break;
+		}
+	}
+	if (text_in_utf8) {
+		strcpy(vty_text, "SMS can't be displayed. See log/sms-file.");
 	}
 	vty_notify(ms, NULL);
 	vty_notify(ms, "SMS from %s: '%s'\n", gsms->address, vty_text);
@@ -285,7 +293,10 @@ static int gsm340_rx_tpdu(struct gsm_trans *trans, struct msgb *msg)
 			gsm_7bit_decode(gsms->text, smsp, gsms->user_data_len);
 			break;
 		case DCS_8BIT_DATA:
+			break;
 		case DCS_UCS2:
+			osmo_ucs2_to_utf8_decode(gsms->text, sizeof(gsms->text), gsms->user_data, gsms->user_data_len / 2);
+			break;
 		case DCS_NONE:
 			break;
 		}
@@ -297,9 +308,8 @@ static int gsm340_rx_tpdu(struct gsm_trans *trans, struct msgb *msg)
 	     sms_mti, gsms->msg_ref,
 	     gsms->protocol_id, gsms->data_coding_scheme, gsms->address,
 	     gsms->user_data_len,
-			sms_alphabet == DCS_7BIT_DEFAULT ? gsms->text :
-				osmo_hexdump(gsms->user_data,
-						gsms->user_data_len));
+			(sms_alphabet == DCS_7BIT_DEFAULT || (sms_alphabet == DCS_UCS2 && get_terminal_utf8_support())) ? 
+				gsms->text : osmo_hexdump(gsms->user_data, gsms->user_data_len));
 
 	switch (sms_mti) {
 	case GSM340_SMS_DELIVER_SC2MS:
-- 
1.7.2.5

